<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2004 Jrme Schneider (typo3dev@ameos.com)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * 'Ameos Formidable'
 *
 * @author	Jrme Schneider <typo3dev@ameos.com>
 */

require_once(t3lib_extmgm::extPath("api_macmade") . "class.tx_apimacmade.php");

require_once(t3lib_extmgm::extPath("ameos_formidable") . "pi1/class.mainvalidator.php");
require_once(t3lib_extmgm::extPath("ameos_formidable") . "pi1/class.maindatahandler.php");
require_once(t3lib_extmgm::extPath("ameos_formidable") . "pi1/class.mainrenderer.php");
require_once(t3lib_extmgm::extPath("ameos_formidable") . "pi1/class.mainrenderlet.php");
require_once(t3lib_extmgm::extPath("ameos_formidable") . "pi1/class.mainactionlet.php");

require_once(t3lib_extmgm::extPath("ameos_formidable") . "mod1/class.formlets_tools.php");

class tx_ameosformidable_pi1
{
	var $prefixId			= "tx_ameosformidable_pi1";					// Same as class name
	var $scriptRelPath		= "pi1/class.tx_ameosformidable_pi1.php";	// Path to this script relative to the extension dir.
	var $extKey				= "ameos_formidable";						// The extension key.

	var $apimacmade_version = "2.0";

	var $_oParent			= null;
	var $_xmlPath			= null;
	var $_xmlData			= null;
	var $_aConf				= null;

	var $_aValidators		= null;
	var $_aDataHandlers		= null;
	var $_aRenderers		= null;
	var $_aRenderlets		= null;
	var $_aActionlets		= null;

	var $oRenderer			= null;
	var $oDataHandler		= null;
	var $aORenderlets		= null;

	var $_dhEdit			= false;
	var $_aValidationErrors = array();

	var $aDebug				= array();
	var $start_tstamp		= null;


	/*********************************
	 *
	 * FORMidable initialization
	 *
	 *********************************/

	/**
	 * Standard init function
	 * Initializes :
	 * - api_macmade
	 * - the reference to the parent Extension ( stored in $this->_oParent )
	 * - the XML conf
	 * - the internal collection of Validators
	 * - the internal collection of DataHandlers
	 * - the internal collection of Renderers
	 * - the internal collection of Renderlets
	 * - the Renderer as configured in the XML conf in the /formidable/control/renderer/ section
	 * - the DataHandler as configured in the XML conf in the /formidable/control/datahandler/ section
	 *
	 * @param	object		Parent extension using FORMidable
	 * @param	string		Absolute path to the XML configuration file
	 * @return	void
	 */
	function init(&$oParent, $xmlPath)
	{
		$this->start_tstamp	= t3lib_div::milliseconds();

		$this->_oParent		=& $oParent;
		$this->_xmlPath		= $xmlPath;

		$this->_loadXmlConf();

		$this->_loadDeclaredValidators();
		$this->_loadDeclaredDataHandlers();
		$this->_loadDeclaredRenderers();
		$this->_loadDeclaredRenderlets();
		$this->_loadDeclaredActionlets();

		$this->_initRenderlets();
		$this->_initRenderer();
		$this->_initDataHandler();
	}

	/**
	 * Initializes the DataHandler used for this FORM
	 *
	 * @return	void
	 * @see	tx_ameosformidable_pi1::init()
	 */
	function _initDataHandler()
	{
		$aConfDataHandler = $this->_navConf("/control/datahandler/");
		$aDataHandler = $this->_getDataHandlerForType($aConfDataHandler["type"]);

		if($aDataHandler)
		{
			$this->oDataHandler = $this->_makeDataHandler($aDataHandler["EXTKEY"]);

			$entryId = $this->oDataHandler->_edition();
			if($entryId !== false)
			{
				$this->oDataHandler->entryId		= $entryId;
				$this->oDataHandler->aStoredData	= $this->oDataHandler->_getStoredData();
			}
		}
		else
		{ $this->mayday("TYPE '" . $aConfDataHandler["type"] . "' is not associated to any DATAHANDLER");}
	}

	/**
	 * Initializes the Renderer used for this FORM
	 *
	 * @return	void
	 * @see	tx_ameosformidable_pi1::init()
	 */
	function _initRenderer()
	{
		$aConfRenderer = $this->_navConf("/control/renderer/");
		$aRenderer = $this->_getRendererForType($aConfRenderer["type"]);

		if($aRenderer)
		{ $this->oRenderer = $this->_makeRenderer($aRenderer["EXTKEY"]);}
		else
		{ $this->mayday("TYPE '" . $aConfRenderer["type"] . "' is not associated to any RENDERER");}
	}

	/**
	 * Initializes the Renderlets used for this FORM
	 *
	 * @return	void
	 * @see	tx_ameosformidable_pi1::init()
	 */
	function _initRenderlets()
	{
		$aElements = $this->_navConf("/elements/");

		if(is_array($aElements))
		{
			while(list(, $aElement) = each($aElements))
			{ $this->aORenderlets[$aElement["elementname"]] = $this->_makeRenderlet($aElement);}
		}
	}

	/**
	 * Loads the internal _aConf configuration array from the XML file
	 * IMPORTANT NOTE : the root /formidable is deleted, so all path shouldn't start with /formidable
	 *
	 * @return	boolean		true if conf is valid, according to tx_ameosformidable_pi1::_checkConf()
	 * @see	tx_ameosformidable_pi1::init()
	 */
	function _loadXmlConf()
	{
		$this->_xmlData = implode("", file($this->_xmlPath));
		$this->_aConf = tx_apimacmade::div_xml2array($this->_xmlData);

		// the root is deleted
		$this->_aConf = $this->_aConf["formidable"];

		return $this->_checkConf();
	}

	/**
	 * Loads the internal collection _aDataHandlers of declared DataHandlers
	 * DataHandlers are declared in DH extension's ext_localconf.php file
	 *  using tx_ameosformidable_pi1::declareDataHandler($_EXTKEY, $TYPE)
	 *  where $TYPE is the service name of this DataHandler
	 *
	 * @return	void
	 * @see	tx_ameosformidable_pi1::init(), tx_ameosformidable_pi1::declareDataHandler()
	 */
	function _loadDeclaredDataHandlers()
	{
		$this->_aDataHandlers = $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["datahandlers"];
		reset($this->_aDataHandlers);
	}

	/**
	 * Loads the internal collection _aRenderlets of declared Renderlets
	 * Renderlets are declared in RDT extension's ext_localconf.php file
	 *  using tx_ameosformidable_pi1::declareRenderlet($_EXTKEY, $TYPE)
	 *  where $TYPE is the service name of this Renderlet
	 *
	 * @return	void
	 * @see	tx_ameosformidable_pi1::init(), tx_ameosformidable_pi1::declareRenderlet()
	 */
	function _loadDeclaredRenderlets()
	{
		$this->_aRenderlets = $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["renderlets"];
		reset($this->_aRenderlets);
	}

	/**
	 * Loads the internal collection _aRenderers of declared Renderers
	 * Renderers are declared in RDR extension's ext_localconf.php file
	 *  using tx_ameosformidable_pi1::declareRenderer($_EXTKEY, $TYPE)
	 *  where $TYPE is the service name of this Renderer
	 *
	 * @return	void
	 * @see	tx_ameosformidable_pi1::init(), tx_ameosformidable_pi1::declareRenderer()
	 */
	function _loadDeclaredRenderers()
	{
		$this->_aRenderers = $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["renderers"];
		reset($this->_aRenderers);
	}

	/**
	 * Loads the internal collection _aValidators of declared Validators
	 * Validators are declared in VA extension's ext_localconf.php file
	 *  using tx_ameosformidable_pi1::declareValidator($_EXTKEY, $TYPE)
	 *  where $TYPE is the service name of this Validator
	 *
	 * @return	void
	 * @see	tx_ameosformidable_pi1::init(), tx_ameosformidable_pi1::declareValidator()
	 */
	function _loadDeclaredValidators()
	{
		$this->_aValidators = $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["validators"];
		reset($this->_aValidators);
	}

	/**
	 * Loads the internal collection _aActionlets of declared Actionlets
	 * Actionlets are declared in ACTs extension's ext_localconf.php file
	 *  using tx_ameosformidable_pi1::declareActionlet($_EXTKEY, $TYPE)
	 *  where $TYPE is the service name of this Actionlet
	 *
	 * @return	void
	 * @see	tx_ameosformidable_pi1::init(), tx_ameosformidable_pi1::declareActionlet()
	 */
	function _loadDeclaredActionlets()
	{
		$this->_aActionlets = $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["actionlets"];
		reset($this->_aActionlets);
	}

	/**
	 * Checks if the conf given in the XML file is complete and valid
	 * ! NOT COMPLETE !
	 * TODO : perform XML schema validation
	 *
	 * @return	boolean		TRUE if valid, FALSE if not
	 * @see	tx_ameosformidable_pi1::_loadXmlConf()
	 */
	function _checkConf()
	{
		$valid = true;

		if($this->_navConf("/control"))
		{
			// the control section exists

			$name = $this->_navConf("/control/name");

			if(!$name || trim($name) == "")
			{ $valid = false;}

			if(!$this->_navConf("/control/datahandler/type"))
			{ $valid = false;}
		}

		if($this->_navConf("/elements"))
		{
			// on vrifie la section elements

			if(!$this->_navConf("/elements/element"))
			{ $valid = false;}
		}

		return $valid;
	}

	/**
	 * Navigates in the XML configuration of FORMidable
	 * Returns the XML section corresponding to the given Xpath
	 *
	 * @param	string		$path: the path of the XML section to return
	 * @param	array		$aConf: if set, the array to navigate
	 * @return	mixed		mixed if path is valid and XML section found, FALSE if not
	 */
	function _navConf($path, $aConf = -1)
	{
		if($path{0} == "/")
		{ $path = substr($path, 1);}

		if($path{strlen($path) - 1} == "/")
		{ $path = substr($path, 0, strlen($path) - 1);}

		$aPath = explode("/", $path);
		reset($aPath);

		if($aConf == -1)
		{
			reset($this->_aConf);
			$curZone = $this->_aConf;
		}
		else
		{
			reset($aConf);
			$curZone = $aConf;
		}

		while(list(, $level) = each($aPath))
		{
			if(is_array($curZone) && array_key_exists($level, $curZone))
			{ $curZone = $curZone[$level];}
			else
			{ return false;}
		}

		return $curZone;
	}




	/*********************************
	 *
	 * Rendering functions
	 *
	 *********************************/


	 /**
 * Executes the whole rendering process
 * Validators : Executes the validation process and render the errors
 * DataHandler : Executes the data process as defined in the DataHandler ; renders nothing
 * Renderlets : Exexutes the rendering of each Form Field element as defined in section /formidable/elements/ of the XML conf
 * Renderer : Renders the full layout of the FORM, inserting HTML produced by Validators and Renderlets
 *
 * @return	string		compiled full HTML FORM
 */
	function _render()
	{
		// validation of the renderlets
		$this->_validateElements();

		if(count($this->_aValidationErrors) > 0)
		{ $this->_debug($this->_aValidationErrors, "SOME ELEMENTS ARE NOT VALIDATED", __LINE__, __FILE__);}
		else
		{ $this->_debug("", "ALL ELEMENTS ARE VALIDATED", __LINE__, __FILE__);}

		// the datahandler is executed
		$this->oDataHandler->_doTheMagic();

		// Renderlets are rendered
		$aRendered = $this->_renderElements();

		// the renderer is executed
		$sForm = $this->oRenderer->_render($this->_navConf("/control/renderer/"), $aRendered);

		if($this->oDataHandler->_isSubmitted() && $this->oDataHandler->_allIsValid())
		{ $this->_executeActionlets($aRendered, $sForm);}

		$debug = "";
		if(strtoupper($this->_navConf("/control/debug/")) == "TRUE")
		{ $debug = $this->debug();}

		return $sForm . $debug;
	}


	/**
	 * Renders all the Renderlets elements as defined in XML conf
	 *
	 * @return	array		Array of rendered elements, structured as $elementname => $renderedHTML
	 * @see	tx_ameosformidable_pi1::_render()
	 */
	function _renderElements()
	{ return $this->_renderElementsByPath("/elements/");}

	/**
	 * Render all the Renderlets defined in the corresponding XML conf section
	 *
	 * @param	string		$path: Xpath as used in tx_ameosformidable_pi1::_navConf()
	 * @return	array		Array of rendered elements, structured as $elementname => $renderedHTML
	 * @see	tx_ameosformidable_pi1::_renderElements()
	 */
	function _renderElementsByPath($path)
	{
		$aHtmlElements = array();

		$aElements = $this->_navConf($path);

		if(is_array($aElements))
		{
			while(list(, $aElement) = each($aElements))
			{ $aHtmlElements[$aElement["elementname"]] = $this->_renderElement($aElement["elementname"]);}
		}

		reset($aHtmlElements);

		return $aHtmlElements;
	}

	/**
	 * Renders the given Renderlet
	 *
	 * @param	array		$aElement: details about the Renderlet to render, extracted from XML conf / used in formidable_mainrenderlet::_render()
	 * @return	string		The Rendered HTML
	 * @see	tx_ameosformidable_pi1::_renderElementsByPath()
	 */
	function _renderElement($elementname)
	{
		if(array_key_exists($elementname, $this->aORenderlets))
		{
			$aFormData = $this->oDataHandler->_getFormData();
			return $this->aORenderlets[$elementname]->_render($aFormData);
		}

		return "";
	}






	/*********************************
	 *
	 * Renderlet, Renderer, DataHandler and Validator instanciation functions
	 *
	 *********************************/



	/**
	 * Returns informations about the DataHandler for the given TYPE
	 *
	 * @param	string		$type: TYPE (SERVICE NAME) for the DataHandler as declared using tx_ameosformidable_pi1::declareDataHandler()
	 * @return	array		informations about the DataHandler for the given TYPE
	 * @see	tx_ameosformidable_pi1::_initDataHandler(), tx_ameosformidable_pi1::declareDataHandler()
	 */
	function _getDataHandlerForType($type)
	{
		reset($this->_aDataHandlers);
		while(list(, $aInfos) = each($this->_aDataHandlers))
		{
			if($aInfos["TYPE"] == $type)
			{
				reset($aInfos);
				return $aInfos;
			}
		}

		return false;
	}

	/**
	 * Returns informations about the Renderer for the given TYPE
	 *
	 * @param	string		$type: TYPE (SERVICE NAME) for the Renderer as declared using tx_ameosformidable_pi1::declareRenderer()
	 * @return	array		informations about the Renderer for the given TYPE
	 * @see	tx_ameosformidable_pi1::_initRenderer(), tx_ameosformidable_pi1::declareRenderer()
	 */
	function _getRendererForType($type)
	{
		reset($this->_aRenderers);
		while(list(, $aInfos) = each($this->_aRenderers))
		{
			if($aInfos["TYPE"] == $type)
			{
				reset($aInfos);
				return $aInfos;
			}
		}

		return false;
	}

	/**
	 * Returns informations about the Renderlet for the given TYPE
	 *
	 * @param	string		$type: TYPE (SERVICE NAME) for the Renderlet as declared using tx_ameosformidable_pi1::declareRenderlet()
	 * @return	array		informations about the Renderer for the given TYPE
	 * @see	tx_ameosformidable_pi1::_renderElement(), tx_ameosformidable_pi1::declareRenderlet()
	 */
	function _getRenderletForType($type)
	{
		reset($this->_aRenderlets);
		while(list(, $aInfos) = each($this->_aRenderlets))
		{
			if($aInfos["TYPE"] == $type)
			{
				reset($aInfos);
				return $aInfos;
			}
		}

		return false;
	}

	/**
	 * Returns informations about the Validator for the given TYPE
	 *
	 * @param	string		$type: TYPE (SERVICE NAME) for the Validator as declared using tx_ameosformidable_pi1::declareValidator()
	 * @return	array		informations about the Validator for the given TYPE
	 * @see	tx_ameosformidable_pi1::_validateElement(), tx_ameosformidable_pi1::declareValidator()
	 */
	function _getValidatorForType($type)
	{
		reset($this->_aValidators);
		while(list(, $aInfos) = each($this->_aValidators))
		{
			if($aInfos["TYPE"] == $type)
			{
				reset($aInfos);
				return $aInfos;
			}
		}

		return false;
	}

	/**
	 * Returns informations about the Actionlet for the given TYPE
	 *
	 * @param	string		$type: TYPE (SERVICE NAME) for the Actionlet as declared using tx_ameosformidable_pi1::declareActionlet()
	 * @return	array		informations about the Validator for the given TYPE
	 * @see	tx_ameosformidable_pi1::_validateElement(), tx_ameosformidable_pi1::declareActionlet()
	 */
	function _getActionletForType($type)
	{
		reset($this->_aActionlets);
		while(list(, $aInfos) = each($this->_aActionlets))
		{
			if($aInfos["TYPE"] == $type)
			{
				reset($aInfos);
				return $aInfos;
			}
		}

		return false;
	}

	/**
	 * Instanciates and initializes a DataHandler Object for the given $extkey,
	 *  as returned in the array returned by tx_ameosformidable_pi1::_getDataHandlerForType()
	 *
	 * @param	string		$extkey: the extKey of the Renderer to instanciate
	 * @return	object
	 * @see	tx_ameosformidable_pi1::_initDataHandler()
	 */
	function _makeDataHandler($extkey)
	{
		if(t3lib_extMgm::isLoaded($extkey))
		{
			$className = "tx_" . str_replace("_", "", $extkey) . "_pi1";
			require_once(t3lib_extMgm::extPath($extkey) . "pi1/class." . $className . ".php");

			$oDataHandler = new $className();
			$oDataHandler->_init(&$this);

			return $oDataHandler;
		}
		else
		{ $this->mayday("DATAHANDLER '" . $extkey . "' is not available. Check your XML conf.");}
	}

	/**
	 * Instanciates and initializes a Renderer Object for the given $extkey,
	 *  as returned in the array returned by tx_ameosformidable_pi1::_getRendererForType()
	 *
	 * @param	string		$extkey: the extKey of the Renderer to instanciate
	 * @return	object
	 * @see	tx_ameosformidable_pi1::_initRenderer()
	 */
	function _makeRenderer($extkey)
	{
		if(t3lib_extMgm::isLoaded($extkey))
		{
			$className = "tx_" . str_replace("_", "", $extkey) . "_pi1";
			require_once(t3lib_extMgm::extPath($extkey) . "pi1/class." . $className . ".php");

			$oRenderer = new $className();
			$oRenderer->_init(&$this);

			return $oRenderer;
		}
		else
		{ $this->mayday("RENDERER '" . $extkey . "' isn't available. Check your XML conf");}
	}

	/**
	 * Instanciates and initializes a Renderlet Object for the given $extkey,
	 *  as returned in the array returned by tx_ameosformidable_pi1::_getRenderletForType()
	 *
	 * @param	string		$extkey: the extKey of the Renderlet to instanciate
	 * @return	object
	 * @see	tx_ameosformidable_pi1::_renderElement()
	 */
	function _makeRenderlet($aElement)
	{
		$type = $aElement["type"];
		$aRenderlet = $this->_getRenderletForType($type);

		if(is_array($aRenderlet))
		{
			$extkey = $aRenderlet["EXTKEY"];

			if(t3lib_extMgm::isLoaded($extkey))
			{
				$className = "tx_" . str_replace("_", "", $extkey) . "_pi1";
				require_once(t3lib_extMgm::extPath($extkey) . "pi1/class." . $className . ".php");

				$oRenderlet = new $className();
				$oRenderlet->_init(&$this, $aElement);

				return $oRenderlet;
			}
			else
			{ $this->mayday("RENDERLET '" . $extkey . "' isn't available. Check your XML conf.");}
		}
		else
		{ $this->mayday("TYPE " . $type . " is not associated to any RENDERLET");}
	}

	/**
	 * Instanciates and initializes a Validator Object for the given $extkey,
	 *  as returned in the array returned by tx_ameosformidable_pi1::_getValidatorForType()
	 *
	 * @param	string		$extkey: the extKey of the Validator to instanciate
	 * @return	object
	 * @see	tx_ameosformidable_pi1::_validateElement()
	 */
	function _makeValidator($extkey)
	{
		if(t3lib_extMgm::isLoaded($extkey))
		{
			$className = "tx_" . str_replace("_", "", $extkey) . "_pi1";
			require_once(t3lib_extMgm::extPath($extkey) . "pi1/class." . $className . ".php");

			$oValidator = new $className();
			$oValidator->_init(&$this);

			return $oValidator;
		}
		else
		{ $this->mayday("VALIDATOR '" . $extkey . "' isn't available. Check your XML conf.");}
	}

	/**
	 * Instanciates and initializes an Actionlet Object for the given $extkey,
	 *  as returned in the array returned by tx_ameosformidable_pi1::_getActionletForType()
	 *
	 * @param	string		$extkey: the extKey of the Actionlet to instanciate
	 * @return	object
	 * @see	tx_ameosformidable_pi1::_executeActionlet()
	 */
	function _makeActionlet($aElement)
	{
		$type = $aElement["type"];
		$aObject = $this->_getActionletForType($type);

		if(is_array($aObject))
		{
			$extkey = $aObject["EXTKEY"];

			if(t3lib_extMgm::isLoaded($extkey))
			{
				$className = "tx_" . str_replace("_", "", $extkey) . "_pi1";
				require_once(t3lib_extMgm::extPath($extkey) . "pi1/class." . $className . ".php");

				$oActionlet = new $className();
				$oActionlet->_init(&$this, $aElement);

				return $oActionlet;
			}
			else
			{ $this->mayday("ACTIONLET '" . $extkey . "' isn't available. Check your XML conf.");}
		}
		else
		{ $this->mayday("TYPE " . $type . " is not associated to any ACTIONLET");}
	}



	/*********************************
	 *
	 * Validators calling functions
	 *
	 *********************************/

	/**
	 * Validates data returned by all the Renderlets elements as defined in XML conf
	 *
	 * @return	void		Writes into $this->_aValidationErrors[] using tx_ameosformidable_pi1::_declareValidationError()
	 * @see	tx_ameosformidable_pi1::_render()
	 */
	function _validateElements()
	{ $this->_validateElementsByPath("/elements/");}

	/**
	 * Validates data returned by all the Renderlets elements defined in the corresponding XML conf section
	 *
	 * @param	string		$path: Xpath as used in tx_ameosformidable_pi1::_navConf()
	 * @return	void		Writes into $this->_aValidationErrors[] using tx_ameosformidable_pi1::_declareValidationError()
	 * @see	tx_ameosformidable_pi1::_validateElements()
	 */
	function _validateElementsByPath($path)
	{
		$aElements = $this->_navConf($path);

		if(is_array($aElements))
		{
			while(list(, $aElement) = each($aElements))
			{ $this->_validateElement($aElement);}
		}
	}

	/**
	 * Validates the given Renderlet element
	 *
	 * @param	array		$aElement: details about the Renderlet element to validate, extracted from XML conf / used in formidable_mainvalidator::_doTheMagic()
	 * @return	void		Writes into $this->_aValidationErrors[] using tx_ameosformidable_pi1::_declareValidationError()
	 * @see	tx_ameosformidable_pi1::_validateElementsByPath()
	 */
	function _validateElement($aElement)
	{
		if($aElement)
		{
			$aConfValidators = $this->_navConf("/validators/", $aElement);

			if($aConfValidators)
			{
				while(list(, $aConfValidator) = each($aConfValidators))
				{
					// la section existe
					// on apelle le validateur

					$type = $aConfValidator["type"];
					$aValidator = $this->_getValidatorForType($type);

					if($aValidator)
					{
						$aFormData = $this->oDataHandler->_getFormData();
						$oValidator = $this->_makeValidator($aValidator["EXTKEY"]);
						$oValidator->_doTheMagic($aValidator, $aConfValidator, $aElement, $aFormData);
					}
					else
					{ $this->mayday("TYPE " . $type . " isn't associated with any VALIDATOR");}
				}
			}
		}
	}

	/**
	 * Declares validation error
	 * Used by Validators Objects
	 *
	 * @param	string		$elementname: not used for the moment
	 * @param	string		$key: not used for the moment
	 * @param	string		$message: the error message to display
	 * @return	void		Writes into $this->_aValidationErrors[]
	 */
	function _declareValidationError($elementname, $key, $message)
	{
		$this->_aValidationErrors[] = $message;
	}




	/*********************************
	 *
	 * Renderlet, Renderer, DataHandler and Validator declaration functions
	 *
	 *********************************/



	/**
	 * Declares a Renderlet, associating it's extKey to a unique TYPE ( Renderlet SERVICE NAME )
	 *
	 * @param	string		$extkey: the Renderlet's extension key
	 * @param	string		$type: the TYPE ( SERVICE NAME )
	 * @return	void
	 */
	function declareRenderlet($extkey, $type)
	{
		$GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["renderlets"][$extkey] = array(	"EXTKEY"	=> $extkey,
																									"TYPE"		=> $type
																									);
	}

	/**
	 * Declares a Renderer, associating it's extKey to a unique TYPE ( Renderer SERVICE NAME )
	 *
	 * @param	string		$extkey: the Renderer's extension key
	 * @param	string		$type: the TYPE ( SERVICE NAME )
	 * @return	void
	 */
	function declareRenderer($extkey, $type)
	{
		$GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["renderers"][$extkey] = array(	"EXTKEY"	=> $extkey,
																									"TYPE"		=> $type
																									);
	}

	/**
	 * Declares a DataHandler, associating it's extKey to a unique TYPE ( DataHandler SERVICE NAME )
	 *
	 * @param	string		$extkey: the DataHandler's extension key
	 * @param	string		$type: the TYPE ( SERVICE NAME )
	 * @return	void
	 */
	function declareDataHandler($extkey, $type)
	{
		$GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["datahandlers"][$extkey] = array(	"EXTKEY"	=> $extkey,
																										"TYPE"		=> $type
																										);
	}

	/**
	 * Declares a Validator, associating it's extKey to a unique TYPE ( Validator SERVICE NAME )
	 *
	 * @param	string		$extkey: the Validator's extension key
	 * @param	string		$type: the TYPE ( SERVICE NAME )
	 * @return	void
	 */
	function declareValidator($extkey, $type)
	{
		$GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["validators"][$extkey] = array(		"EXTKEY"	=> $extkey,
																										"TYPE"		=> $type
																										);
	}

	/**
	 * Declares an Actionlet, associating it's extKey to a unique TYPE ( Actionlet SERVICE NAME )
	 *
	 * @param	string		$extkey: the Actionlet's extension key
	 * @param	string		$type: the TYPE ( SERVICE NAME )
	 * @return	void
	 */
	function declareActionlet($extkey, $type)
	{
		$GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["actionlets"][$extkey] = array(		"EXTKEY"	=> $extkey,
																										"TYPE"		=> $type
																										);
	}

	/**
	 * Execute each actionlet declared for this FORM
	 *
	 * @param	array		$aRendered:	array containing the HTML of the rendered renderlets
	 * @param	string		$sForm: the whole FORM html string
	 * @return	void
	 * @see tx_ameosformidable_pi1::_render()
	 */
	function _executeActionlets($aRendered, $sForm)
	{
		$aActionlets = $this->_navConf("/control/actionlets/");

		if(is_array($aActionlets))
		{
			while(list(, $aActionlet) = each($aActionlets))
			{
				if($aActionlet)
				{ $this->_executeActionlet($aActionlet, $aRendered, $sForm);}
			}
		}
	}

	/**
	 * Executes the specific process for this actionlet
	 *
	 * @param	array		$aActionlet: details about the Renderlet element to validate, extracted from XML conf / used in formidable_mainvalidator::_doTheMagic()
	 * @param	array		$aRendered: array containing the HTML of the rendered renderlets
	 * @param	string		$sForm: the whole FORM html string
	 * @return	void
	 * @see tx_ameosformidable_pi1::_executeActionlets()
	 */
	function _executeActionlet($aActionlet, $aRendered, $sForm)
	{
		$oActionlet = $this->_makeActionlet($aActionlet);
		$oActionlet->_doTheMagic($aRendered, $sForm);
	}









	/*********************************
	 *
	 * Debugging functions
	 *
	 *********************************/


	/**
	 * Displays a full debug of :
	 * - the XML conf
	 * - the collection of declared DataHandlers
	 * - the collection of declared Renderers
	 * - the collection of declared Renderlets
	 * - the collection of declared Validators
	 *
	 * Can be called by the parent Extension, or by FORMidable itselves, if the XML conf sets /formidable/control/debug/ to TRUE
	 *
	 * @return	void
	 * @see	tx_ameosformidable_pi1::mayday(), tx_ameosformidable_pi1::_render()
	 */
	function debug()
	{
		$aHtml = array();

		$aHtml[] = "<div style = 'background-color: #bed1f4; padding-left: 10px; padding-top: 5px; padding-bottom: 10px;'>";

		$api_path = str_replace($GLOBALS["_SERVER"]["DOCUMENT_ROOT"], "", t3lib_div::getFileAbsFileName("EXT:ameos_formidable/mod1/doc/class.tx_ameosformidable_pi1.php.html"));

		$aHtml[] = "<a name = 'formidable_debugtop' />";
		$aHtml[] = "<h1>FORMidable debug()</h1>";

		$aHtml[] = "<ul>";
		$aHtml[] = "<li><a href = 'http://typo3.org/documentation/document-library/ameos_formidable/' target = '_blank'>FORMidable user documentation</a></li>";
		$aHtml[] = "<li><a href = '" . $api_path . "' target = '_blank'>FORMidable developper API documentation</a></li>";
		$aHtml[] = "</ul>";
		$aHtml[] = "<br>";
		$aHtml[] = "<ul>";
		$aHtml[] = "<li><a href = '#formidable_xmlpath' target = '_self'>XML Path</a></li>";
		$aHtml[] = "<li><a href = '#formidable_xmlfile' target = '_self'>The XML file</a></li>";
		$aHtml[] = "<li><a href = '#formidable_configuration' target = '_self'>FORM configuration</a></li>";
		$aHtml[] = "<li><a href = '#formidable_callstack' target = '_self'>Call stack</a></li>";
		$aHtml[] = "<li><a href = '#formidable_objects' target = '_self'>Available DataHandlers, Renderers, Renderlets, Validators and Actionlets</a></li>";
		$aHtml[] = "</ul>";

		$aHtml[] = "<a name = 'formidable_xmlpath' />";
		$aHtml[] = "<h2>XML Path</h2>";
		$aHtml[] = $this->_xmlPath;

		$aHtml[] = "<p align = 'right'><a href = '#formidable_debugtop' target = '_self'>^top^</a></p>";




		$aHtml[] = "<a name = 'formidable_xmlfile' />";
		$aHtml[] = "<h2>The XML file</h2>";
		$aHtml[] = "<iframe width = '100%' height = '400px' src = '" . str_replace($GLOBALS["_SERVER"]["DOCUMENT_ROOT"], "", $this->_xmlPath) . "' style = 'border:0px solid'></iframe>";

		$aHtml[] = "<p align = 'right'><a href = '#formidable_debugtop' target = '_self'>^top^</a></p>";





		$aHtml[] = "<a name = 'formidable_configuration' />";
		$aHtml[] = "<h2>FORM configuration</h2>";
		$aHtml[] = "<div WIDTH = '100%' style = 'HEIGHT: 400px; overflow: scroll'>" . t3lib_div::view_array($this->_aConf) . "</div>";




		$aHtml[] = "<p align = 'right'><a href = '#formidable_debugtop' target = '_self'>^top^</a></p>";
		
		$aHtml[] = "<a name = 'formidable_callstack' />";
		$aHtml[] = "<h2>Call stack</h2>";
		$aHtml[] = implode("<hr>", $this->aDebug);



		$aHtml[] = "<p align = 'right'><a href = '#formidable_debugtop' target = '_self'>^top^</a></p>";
		
		$aHtml[] = "<a name = 'formidable_objects' />";
		$aHtml[] = "<h2>Available FORMidable Objects</h2>";
		$oTools = new formlets_tools();
		$aHtml[] = $oTools->_renderInfos();

		$aHtml[] = "</div>";

		$aHtml[] = "<p align = 'right'><a href = '#formidable_debugtop' target = '_self'>^top^</a></p>";


		return implode("\n", $aHtml);
	}

	/**
	 * Internal debug function
	 * Calls the TYPO3 debug function if the XML conf sets /formidable/control/debug/ to TRUE
	 *
	 * @param	mixed		$variable: the variable to dump
	 * @param	string		$name: title of this debug section
	 * @param	string		$line: PHP code line calling this function ( __LINE__ )
	 * @param	string		$file: PHP script calling this function ( __FILE__ )
	 * @param	integer		$recursiveDepth: number of levels to debug, if recursive variable
	 * @param	string		$debugLevel: the sensibility of this warning
	 * @return	void
	 */
	function _debug($variable, $name='*variable*', $line='*line*', $file='*file*', $recursiveDepth=6, $debugLevel=E_DEBUG)
	{
		$numcall = sizeof($this->aDebug) + 1;

		$aDebug = array();
		$aDebug[] = "<p align = 'right'><a href = '#formidable_debugtop' target = '_self'>^top^</a></p>";
		$aDebug[] = "<a name = 'formidable_call" . $numcall . "' />";
		$aDebug[] = "<a href = '#formidable_call" . ($numcall - 1) . "'>&lt;&lt; prev</a> / <a href = '#formidable_call" . ($numcall + 1) . "'>next &gt;&gt;</a><br>";
		$aDebug[] = "<strong>#" . $numcall ." - " . $name . "</strong><br>";
		$aDebug[] = "In : " . str_replace(PATH_site, "/", $file) . ":" . $line . "<br>";
		$aDebug[] = "Exec. time : " . round(t3lib_div::milliseconds() - $this->start_tstamp, 4) / 100 ." sec<br>";
		$aDebug[] = t3lib_div::view_array($variable);

		$this->aDebug[] = implode("", $aDebug);
	}

	/**
	 * Stops Formidable and PHP execution : die() if some critical error appeared
	 *
	 * @param	string		$msg: the error message
	 * @return	void
	 */
	function mayday($msg)
	{
		$content	=	"<h1 style = 'color: red'>FORMidable : Mayday</h1>";
		$content	.=	$msg;
		$content	.=	"<hr>";
		$content	.=	$this->debug();

		die($content);
	}



	/*********************************
	 *
	 * Utilitary functions
	 *
	 *********************************/

	/**
	 * Returns the translated string for the given LLL path
	 *
	 * @param	string		$label: LLL path
	 * @return	string		The translated string
	 */
	function _getLLLabel($label)
	{
		if(TYPO3_MODE == "FE")
		{
			// front end
			return $GLOBALS["TSFE"]->sL($label);
		}
		else
		{
			// back end
			return $GLOBALS["LANG"]->getLL($label);
		}
	}

	/**
	 * Parses a template
	 *
	 * @param	string		$templatePath: the path to the template file
	 * @param	string		$templateMarker: the marker subpart
	 * @param	array		$aTags: array containing the values to render
	 * @return	string		HTML string with substituted values
	 */
	function _parseTemplate($templatePath, $templateMarker, $aTags = array())
	{
		// $tempUrl : the path of the template for use with t3lib_div::getUrl()
		// $tempMarker :  the template subpart marker
		// $aTags : the marker array for substitution

		$templateCode = t3lib_div::getUrl($templatePath);
		$template = t3lib_parsehtml::getSubpart($templateCode, $templateMarker);

		reset($aTags);
		while(list($elementname, $valeur) = each($aTags))
		{
			$tag = "{" . $elementname . "}";
			$template = str_replace($tag, $valeur, $template);
		}

		return ereg_replace("\{[a-zA-Z0-9_]*}", "", $template);
	}
}

?>